package Echiquier;

import Echiquier.Modele.Case;
import Echiquier.Modele.ColorJoueur;
import Echiquier.Modele.EchiquierException;
import Echiquier.Modele.Piece.*;
import Echiquier.Vue.IHM;

/**
 * La simulation de l'echiquier
 */
public class Echiquier {
    public static final int MAXLIG = 8; //nombre max de lignes
    public static final int MAXCOL = 8; //nombre max de colonnes
    private Case[][] table;

    public Echiquier(){
        try {
            echiquierInit();
        }catch (EchiquierException e){}

    }

    //echiquer init
    private void echiquierInit() throws EchiquierException {
        table = new Case[MAXLIG][MAXCOL];

        ColorJoueur colorJoueur = null;
        Piece piece = null;

        for(int lig=0; lig<MAXLIG; lig++) {
            if(lig <= 1){
                colorJoueur = ColorJoueur.NOIR;
            }else {
                colorJoueur = ColorJoueur.BLANCHE;
            }
            for (int col = 0; col < MAXCOL; col++) {
                table[lig][col] = new Case(lig,col);
                //ajouter les pieces dans l'echiquier
                if(lig <= 1 || lig >= 6) {
                    //ajouter les pions
                    if(lig == 1 || lig == 6){
                        piece = new Pion(colorJoueur);
                    }
                    else {
                        //ajouter les tours
                        if(col == 0 || col == 7){
                            piece = new Tour(colorJoueur);
                        }
                        //ajouter les cavaliers
                        if(col == 1 || col == 6){
                            piece = new Cavalier(colorJoueur);
                        }
                        //ajouter les fous
                        if(col == 2 || col == 5){
                            piece = new Fou(colorJoueur);
                        }
                        //ajouter les Dames
                        if(col == 3){
                            piece = new Dame(colorJoueur);
                        }
                        //ajouter les rois
                        if(col == 4){
                            piece = new Roi(colorJoueur);
                        }
                    }

                    piece.setEchiquier(this);
                    table[lig][col].setPiece(piece);
                }
            }
        }
        System.out.println("L'echiquier init");
    }

    /*
    public Case[] deplaceDisponible(Case depart) throws EchiquierException{
        if(! depart.estEuPiece()) {
            throw new EchiquierException("Le case n'a pas un piece !");
        }
        Case[] disponible = new Case[Echiquier.MAXCOL*Echiquier.MAXLIG];
        int tmpCase = 0;
        for(int lig=0; lig<Echiquier.MAXLIG; lig++){
            for(int col=0; col<Echiquier.MAXCOL; col++){
                if(depart.getPiece().estDpValide(depart, table[lig][col])){
                    disponible[tmpCase++] = table[lig][col];
                }
            }
        }
        return disponible;
    }
    */

    public void deplace(Case[] position) throws EchiquierException {
        //recupperer la Case dans l'echiquier
        Case depart = getCase(position[0]);
        Case arrive = getCase(position[1]);
        if(depart.estEuPiece()) {
            if (depart.getPiece().estDpValide(depart,arrive)) {
                //mettre le piece dans le case
                arrive.setPiece(depart.getPiece());
                depart.setPiece(null);
            } else {
                throw new EchiquierException("Il n'est pas possible de déplacer la pièce en " + depart.toString() + " vers " + arrive.toString() + "!");
            }
        }else {
            throw new EchiquierException("Le case" + depart.toString() +  " n'a pas un piece !");
        }

    }

    public Case[][] getTable() {
        return table;
    }

    public Case getCase(Case unCase){
        return table[unCase.getLig()][unCase.getCol()];
    }

    public Case getCase(int lig, int col){
        return table[lig][col];
    }

    public static void main(String[] argv){
        new IHM(new Echiquier());
    }


    public boolean estBloqueDiagonales(Case depart, Case arrvie){
        if(estEnDiagonales(depart,arrvie)){
            //Si l'arrive est en haut à droite
            int nb = depart.getLig() - arrvie.getLig();
            if(depart.getLig()-arrvie.getLig()>0 && depart.getCol()-arrvie.getCol()>0){
                for(int i=1;i<nb; i++){
                    if(getCase(depart.getLig()+i,depart.getCol()+i).estEuPiece())
                        return false;
                }
            }

            //Si l'arrive est en haut à gauche
            if(depart.getLig()-arrvie.getLig()>0 && depart.getCol()-arrvie.getCol()<0){
                for(int i=1;i<nb; i++){
                    if(getCase(depart.getLig()+i,depart.getCol()-i).estEuPiece())
                        return false;
                }
            }

            //Si l'arrive est en base à gauche
            if(depart.getLig()-arrvie.getLig()<0 && depart.getCol()-arrvie.getCol()<0){
                for(int i=1;i<nb; i++){
                    if(getCase(depart.getLig()-i,depart.getCol()-i).estEuPiece())
                        return false;
                }
            }

            //Si l'arrive est en base à droite
            if(depart.getLig()-arrvie.getLig()<0 && depart.getCol()-arrvie.getCol()>0){
                for(int i=1;i<nb; i++){
                    if(getCase(depart.getLig()-i,depart.getCol()+i).estEuPiece())
                        return false;
                }
            }
        }else
            return false;
        return true;
    }

    public boolean estBloqueDroit(Case depart, Case arrive){
        if(estEnLigne(depart,arrive)){
            int nbCol = depart.getCol() - arrive.getCol();
            int nbLig = depart.getLig() - arrive.getLig();

            //La deplacement horizontal
            if(depart.getLig() == arrive.getLig()){
                //A droit
                if(nbCol>0){
                    for (int i=0;i<nbCol;i++){
                        if(getCase(depart.getLig(), depart.getCol() + i).estEuPiece()){
                            return false;
                        }
                    }
                }

                //A gauche
                if(nbCol>0){
                    for (int i=0;i<nbCol;i++){
                        if(getCase(depart.getLig(), depart.getCol() - i).estEuPiece()){
                            return false;
                        }
                    }
                }
            }

            //La deplacement vertical
            if(depart.getCol() == depart.getCol()){
                //En haut
                if(nbLig>0){
                    for (int i=0;i<nbLig; i++){
                        if(getCase(depart.getLig() + i, depart.getCol()).estEuPiece()){
                            return false;
                        }
                    }
                }
                //En base
                if(nbLig>0){
                    for (int i=0;i<nbLig; i++){
                        if(getCase(depart.getLig() - i, depart.getCol()).estEuPiece()){
                            return false;
                        }
                    }
                }
            }
        }else
            return false;
        return true;
    }

    private boolean estEnLigne(Case depart,Case arrive){
        return depart.getLig() == arrive.getLig() || depart.getCol() == arrive.getCol();
    }

    //Verifier le deplacement est diagonales
    private boolean estEnDiagonales(Case depart, Case arrvie){
        return depart.getCol()-arrvie.getCol() == depart.getLig() - arrvie.getLig();
    }

}
